// ********************************************
// *			         	      *
// * TEST PER split_region() e region_merge() *
// *			         	      *
// ********************************************

#include <CGAL/Cartesian.h>
#include <CGAL/Homogeneous.h>

// Rappresentazione dei punti:
typedef CGAL_Cartesian<double> Cdouble;

#include <peg2_info.h>
#include <peg2.h>

// Per comodita' ...
typedef vertex_2_info<Cdouble,NIL> v_info;
typedef edge_2_info<NIL> e_info;
typedef face_2_info<Cdouble,NIL> f_info;
typedef PEG2<v_info,e_info,f_info> Gtype;

void handle(bool ok_flag)
{
  if (ok_flag)
    cout << "ok" << endl;
  else
  {
    cout << "error" << endl;
    exit(0);
  }
}

main()
{
  Gtype G1;

  Gtype::Vertex_2 v0(0,0), v1(3,0), v2(1,1), v3(2,1), v4(10,10); 
  Gtype::Vertex_2 *pV0, *pV1, *pV2, *pV3, *pV4;
  Gtype::Edge_2 *pE0, *pE1, *pE2, *pE3, *pE4, *pE5;
  Gtype::Face_2 *pF0, *pF1, *pF2;

  cout << "add infinity face ";
  handle(G1.add_infinity_face(pF0));
  cout << "add point feature ";
  handle(G1.add_point_feature(pF0,v4,pV4));
  cout << "add point feature ";
  handle(G1.add_point_feature(pF0,v0,pV0));
  cout << "add point feature ";
  handle(G1.add_point_feature(pF0,v1,pV1));
  cout << "add line feature ";
  handle(G1.add_line_feature(pV0,pV1,NULL,NULL,pE0));
  cout << "expand line to region ";
  handle(G1.expand_line_to_region(pE0,pF1,pE1));
  cout << "add point feature ";
  handle(G1.add_point_feature(pF1,v2,pV2));
  cout << "add point feature ";
  handle(G1.add_point_feature(pF1,v3,pV3));
  cout << "add line feature ";
  handle(G1.add_line_feature(pV0,pV2,pE0,NULL,pE2));
  cout << "add line feature ";
  handle(G1.add_line_feature(pV1,pV3,pE1,NULL,pE3));
  cout << "add line feature ";
  handle(G1.add_line_feature(pV1,pV4,pE0,NULL,pE4));
  cout << "add line feature (error) ";
  handle(!G1.add_line_feature(pV2,pV3,pE2,pE3,pE5));
  cout << "add line feature (error) ";
  handle(!G1.add_line_feature(pV3,pV4,pE3,NULL,pE5));
  cout << "split region ";
  handle(G1.split_region(pF1,pV2,pV3,pE5,pF2));
  G1.print_subdivision();
  
  cout << "line abstraction (error) ";
  handle(!G1.line_abstraction(pE3));
  cout << "region merge ";
  handle(G1.region_merge(pE5,pF2));
  G1.print_subdivision();
  
  cout << "F-Face-Vertex :" << pF2 << endl;
  Gtype::Vertex_List vl;
  G1.f_face_vertex(pF2, vl);
  vl.print();
  vl.free();
  
  cout << "F-Face-Edge :" << endl;
  Gtype::Edge_List_List ell;
  G1.f_face_edge(pF2, ell);
  for (Gtype::Edge_List_Iterator eli = ell.begin(); eli != ell.end(); eli++)
    (*eli).print();
  ell.free();
  
  cout << "Face-Vertex :" << endl;
  Gtype::Vertex_List_List vll;
  G1.face_vertex(pF2, vll);
  for (Gtype::Vertex_List_Iterator vli = vll.begin(); vli != vll.end(); vli++)
    (*vli).print();
  vll.free();
  
  cout << "Face-Edge :" << endl;
  G1.face_edge(pF2, ell);
  for (eli = ell.begin(); eli != ell.end(); eli++)
    (*eli).print();
  ell.free();
  
  cout << "Face-Face :" << endl;
  Gtype::Face_List_List fll;
  G1.face_face(pF2, fll);
  for (Gtype::Face_List_Iterator fli = fll.begin(); fli != fll.end(); fli++)
    (*fli).print();
  fll.free();  
}

