// ****************************************************************************
//
// Il modulo definisce le funzioni private e pubbliche necessarie alla
// implementazione delle funzioni di interfaccia al tipo PEG2.
//
// ****************************************************************************

// ********************* CREATE POINT *****************************************  
template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
create_point(v_type *& V_ptr, v_type V)
{
  V_ptr = new v_type;
  * V_ptr = V;
  V_ptr->assign_iterator_pointer(vertex_list.push(V_ptr));
}

// ********************* CREATE LINE ******************************************
template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
create_line(v_type * V1_ptr, v_type * V2_ptr, e_type *& E_ptr)
{
  if ((V1_ptr != NULL) && (V2_ptr != NULL))
  {
    E_ptr = new e_type;
    E_ptr->assign_source_vertex(V1_ptr);
    E_ptr->assign_target_vertex(V2_ptr);
    E_ptr->assign_iterator_pointer(edge_list.push(E_ptr));
  }
}

// ********************* CREATE FACE ******************************************
template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
create_face(e_type * E_ptr, f_type *& F_ptr)
{
  if (E_ptr != NULL)
  {
    F_ptr = new f_type;
    F_ptr->assign_bounding_edge(E_ptr);
    F_ptr->assign_iterator_pointer(face_list.push(F_ptr));
  }
}

// *************** ADD INFINITY FACE ******************************************
// Inserisce la faccia infinita nella suddivisione e la assegna a F_ptr. In
// caso di errore (la faccia infinita esiste gia') F_ptr e' assegnato a NULL.
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
add_infinity_face(f_type *& F_ptr)
{
  F_ptr = NULL;
  
  if (infinity_face != NULL)
    return false;
    
  F_ptr = new(f_type);
  infinity_face = F_ptr;
  F_ptr->assign_iterator_pointer(face_list.push(F_ptr));
  
  return true;
}

// *************** ADD POINT FEATURE ******************************************
// Inserisce il vertice V nelle feature di F_ptr e lo assegna a V_ptr. In caso
// di errore V_ptr e' assegnato a NULL.
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
add_point_feature(f_type * F_ptr, v_type V, v_type *& V_ptr)
{
  V_ptr = NULL;
  
  if (F_ptr == NULL)
    return false;
    
  create_point(V_ptr,V);
  V_ptr->assign_containing_face(F_ptr);
  F_ptr->add_f_vertex(V_ptr);
  
  return true;
}

// *************** POINT ABSTRACTION ******************************************
// Elimina il vertice V_ptr dalla suddivisione. V_ptr deve essere un
// vertice isolato. In caso di successo V_ptr viene reinizializzato a NULL.
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
point_abstraction(v_type *& V_ptr)
{
  if (V_ptr == NULL)
    return false;
  if (!V_ptr->is_isolated())
    return false;
  f_type * f = V_ptr->containing_face();
  if (f == NULL)
    return false;
    
  // Elimina il vertice dalla lista dei punti isolati contenuti in f
  vi_type V_pos;
  if (!f->is_f_vertex(V_ptr,V_pos))
    return false;
  f->remove_f_vertex(V_pos);
     
  // elimina il vertice dalla suddivisione
  remove_entity(V_ptr);
  
  V_ptr = NULL;

  return true;
}

// *************** ADD LINE FEATURE *******************************************
// Inserisce la nuova linea di estremi V1_ptr e V2_ptr alla suddivisione. Allo
// scopo controlla che i vertici siano distinti, che non si formino nuove
// regioni e che la linea non unisca punti appartenenti a regioni diverse nel
// caso che almeno uno dei due sia interno a qualche regione.
// La nuova linea rappresentata da E_ptr e' inserita in modo che E_ptr risulti
// essere lo spigolo ottenuto applicando la relazione EE di E1_ptr in V1 e di
// E2_ptr in V2.
// E1_ptr ed E2_ptr devono appartenere rispettivamente ai vertici incidenti
// in V1_ptr e V2_ptr o essere NULL nel caso di vertice isolato

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
add_line_feature(v_type * V1_ptr, v_type * V2_ptr, e_type * E1_ptr, 
                 e_type * E2_ptr, e_type *& E_ptr)
{
  E_ptr = NULL;

  if ((V1_ptr  == NULL) || (V2_ptr == NULL) || (V1_ptr == V2_ptr))
    return false;
   
   // Controlla che E1_ptr ed E2_ptr siano incidenti in V1_ptr e V2_ptr 
  if (E1_ptr == NULL)
  {
    if (!V1_ptr->is_isolated())
      return false;
  }
  else
  { 
    if (!E1_ptr->is_incident(V1_ptr))
       return false;
  }
  if (E2_ptr == NULL)
  {
    if (!V2_ptr->is_isolated())
      return false;
  }
  else
  { 
    if (!E2_ptr->is_incident(V2_ptr))
       return false;
  }
  
  // se i vertici sono gia' connessi si creerebbe una nuova faccia
  if (connected_vertices(V1_ptr,V2_ptr))
    return false;
    
  f_type * F_ptr = NULL;
  f_list_type fl1, fl2;
  vertex_face(V1_ptr, fl1);
  vertex_face(V2_ptr, fl2);
  if (fl1.empty() || fl2.empty())
    return false;
  if (!is_on_some_boundary(V1_ptr)) // V1_ptr interno a qualche faccia
  {
    F_ptr = fl1.front();
    if (!is_on_some_boundary(V2_ptr)) // V2_ptr interno a qualche faccia
    {
      // se entrambi i vertici sono interni ad una faccia, la faccia deve
      // essere la medesima
      if (F_ptr != fl2.front())
        return false;
    }
    else // V2_ptr sul contorno di qualche faccia;
    {
      // tra le facce che hanno V2_ptr sul contorno deve esserci anche quella
      // contenente V1_ptr
      if (!fl2.is_in_list(F_ptr))
        return false;
      
      // non si possono aggiungere spigoli feature che hanno un vertice sul
      // contorno di un cappio che li contiene
      if (!is_infinity_face(F_ptr))
      {
        if (F_ptr->bounding_edge()->is_closed())
          return false;
      }
    }
  }
  else  // V1 sul contorno di qualche faccia
  {
    if (!is_on_some_boundary(V2_ptr)) // V2_ptr interno a qualche faccia
    {
      // tra le facce che hanno V1_ptr sul contorno deve esserci anche quella
      // contenente V2_ptr
      F_ptr = fl2.front();
      if (!fl1.is_in_list(F_ptr))
        return false;
        
      // non si possono aggiungere spigoli feature che hanno un vertice sul
      // contorno di un cappio che li contiene
      if (!is_infinity_face(F_ptr))
      {
        if (F_ptr->bounding_edge()->is_closed())
          return false;
      }
    }
    else // V2_ptr sul contorno di qualche faccia;
    {
      // Calcola la faccia in comune ai due vertici (N.B. poiche' i due vertici
      // non sono connessi tale faccia puo' essere al piu' unica).
      fi_type fi_end = fl1.end();
      for (fi_type fi = fl1.begin(); fi != fi_end; fi++)
      {
        F_ptr = * fi;
        if (fl2.is_in_list(F_ptr))
          break;
      }
      if (fi == fi_end)
        return false;
        
      // non si possono aggiungere spigoli feature che hanno un vertice sul
      // contorno di un cappio che li contiene
      if (!is_infinity_face(F_ptr))
      {
        e_type * e = F_ptr->bounding_edge();
        if ((e->is_closed()) && 
            ((e->source_vertex() == V1_ptr) || (e->source_vertex() == V2_ptr)))
          return false;
      }
    }
  }

  // poiche' nuovo spigolo da creare deve essere inserito in modo che la
  // relazione EE di E1_ptr in V1_ptr e E2_ptr in V2_ptr siano ad esso
  // associate, controlla che E1_ptr e E2_ptr siano adatti allo scopo
  // Sono gestiti anche i cappi in senso antiorario.
  if (E1_ptr != NULL)
  {
    if (E1_ptr->counter_clock_face(V1_ptr) != F_ptr)
      return false;  
  }
  if (E2_ptr != NULL)
  {
    if (E2_ptr->counter_clock_face(V2_ptr) != F_ptr)
      return false;  
  }
    
  // creo la linea E e assegno V1 e V2 come estremi
  create_line(V1_ptr, V2_ptr, E_ptr);

  ei_type ei1, ei2;
  if (is_on_feature_edge(V1_ptr, F_ptr, ei1))
  {
    if (is_on_feature_edge(V2_ptr, F_ptr, ei2))
    {
      // dato che le feature vengono connesse elimina uno dei rappresentanti 
      F_ptr->remove_f_inside_edge(ei1);
    }
  }
  else // V1_ptr non appartiene ad una feature di F_ptr
  {
    if (!is_on_feature_edge(V2_ptr, F_ptr, ei2))
    {
      // Viene aggiunta una feature, inserisce il rappresentante 
      F_ptr->add_f_inside_edge(E_ptr);
    }
  }
      
  // relazione VE di V1_ptr ed EE di E_ptr e degli eventuali vertici incidenti
  // in V1_ptr
  if (V1_ptr->is_isolated())
  {
    V1_ptr->assign_incident_edge(E_ptr);
    E_ptr->assign_source_edge(E_ptr);
  }
  else
  {
    e_type * e = E1_ptr->edge_incident_in_vertex(V1_ptr);
    E_ptr->assign_source_edge(e); 
    
    E1_ptr->assign_edge_incident_in_vertex(V1_ptr, E_ptr);
  }

  // relazione VE di V2_ptr ed EE di E_ptr e degli eventuali vertici incidenti
  // in V2_ptr
  if (V2_ptr->is_isolated())
  {
    V2_ptr->assign_incident_edge(E_ptr);
    E_ptr->assign_target_edge(E_ptr);
  }
  else
  {
    e_type * e = E2_ptr->edge_incident_in_vertex(V2_ptr);
    E_ptr->assign_target_edge(e); 
    
    E2_ptr->assign_edge_incident_in_vertex(V2_ptr, E_ptr);
  }

  // aggiorna le relazioni EF
  E_ptr->assign_left_face(F_ptr);
  E_ptr->assign_right_face(F_ptr);
  
  return true;
}


// *************** LINE ABSTRACTION *******************************************
// Elimina lo spigolo E_ptr dalla suddivisione. In caso di successo
// reinizializza E_ptr a NULL.
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
line_abstraction(e_type *& E_ptr)
{  
   if (E_ptr == NULL)
     return false;
   
  // controlla che E_ptr appartenga ad una feature ed assegna a es ed et
  // rispettivamente gli spigoli incidenti nel vertice source e nel vertice
  // target di E_ptr, ad esso successivi (in senso antiorario) e appartenenti
  // alla medesima feature
  e_type * es, * et;
  if (!get_next_feature_edges(E_ptr, es, et))
    return false;
    
  f_type * f = E_ptr->left_face();

  // memorizza la relazione Edge-Edge totale
  e_type * e00, * e01, * e10, * e11;
  edge_edge(E_ptr, e00, e01, e10, e11);
   
  // aggiorna le relazioni EE degli spigoli incidenti in E_ptr, le relazioni
  // VF e VE* di v0 e v1 (se necessario) e la relazione FV di f (se necessario)
  v_type * v0 = E_ptr->source_vertex(), * v1 = E_ptr->target_vertex();
  if (e01 != E_ptr)
  {
    e01->assign_edge_incident_in_vertex(v0, e00);
      
    // nel caso E_ptr fosse stato il rappresentante della relazione VE* di v0
    // assegna un nuovo rappresentante
    if (v0->incident_edge() == E_ptr)
      v0->assign_incident_edge(e01);
  }
  else // v0 diventa un vertice isolato
  {
    v0->assign_containing_face(f);
    v0->assign_incident_edge(NULL);
  }
  
  if (e10 != E_ptr)
  {
    e10->assign_edge_incident_in_vertex(v1, e11);
      
    // nel caso E_ptr fosse stato il rappresentante della relazione VE* di v1
    // assegna un nuovo rappresentante
    if (v1->incident_edge() == E_ptr)
      v1->assign_incident_edge(e10);    
  }
  else // v1 diventa un vertice isolato
  {
    v1->assign_containing_face(f);
    v1->assign_incident_edge(NULL);
  }
    
  // aggiorna la relazione FIE di f
  update_feature_rel(f, E_ptr, es, et);

  // cancella l'edge dalla lista della suddivisione
  remove_entity(E_ptr);

  E_ptr = NULL;
  
  return true;
}


// *************** EXPAND POINT TO LINE ***************************************
// Espande un punto in una linea. Allo scopo:
// - V_ptr rappresenta il punto da espandere. Esso viene eliminato e sostituito
//   dai due nuovi punti V1 e V2
// - E1_ptr ed E2_ptr rappresentano gli spigoli da agganciare alla linea.
//   Essi devono essere incidenti in V_ptr. Tutti gli spigoli incidenti in 
//   V_ptr e compresi (in senso orario) tra E1_ptr (incluso) ed E2_ptr
//   (escluso) avranno come nuovo vertice V1 gli altri V2. Se E1_ptr == NULL
//   tutti gli spigoli avranno come nuovo vertice V2, se E2_ptr == NULL tutti
//   gli spigoli avranno come nuovo vertice V1. Se E1_ptr == E2_ptr == NULL, il
//   vertice V_ptr deve essere isolato.
//   La nuova linea rappresentata da E_ptr e' inserita in modo che E_ptr
//   risulti essere lo spigolo ottenuto applicando la relazione EE di 
//   E1_ptr in V1 e di E2_ptr in V2
// - V1_ptr e V2_ptr vengono assegnati a rappresentare rispettivamente  V1 e V2
// - E_ptr viene assegnato a rappresentare la nuova linea.

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
expand_point_to_line(v_type * V_ptr, e_type * E1_ptr, e_type * E2_ptr,
		     v_type V1, v_type *& pV1, v_type V2,
		     v_type *& pV2, e_type *& E_ptr)
{
  E_ptr = NULL;
   
  if (V_ptr == NULL)
    return false;
    
  if ((E1_ptr == E2_ptr) && (E1_ptr != NULL))
    return false;
    
  if (E1_ptr != NULL)
    if (E1_ptr->is_closed())
      return false;
  if (E2_ptr != NULL)
    if (E2_ptr->is_closed())
      return false;

  if ((V_ptr->is_isolated()) && ((E1_ptr != NULL)||(E2_ptr != NULL)))
    return false;

  if ((!(V_ptr->is_isolated())) && ((E1_ptr == NULL)&&(E2_ptr == NULL)))
    return false;

  // lista degli spigoli associati alla relazione Vertex-Edge totale di V_ptr
  e_list_type el;
  vertex_edge(V_ptr, el);
 
  // controlla la corretezza degli spigoli passati in input  
  ei_type ei1,ei2;
  // controlla che E1_ptr sia associato alla relazione Vertex-Edge totale di
  // V_ptr e ne memorizza la posizione all'interno della lista (in ei1).
  // Se E1_ptr == NULL ei1 e' assegnato al past-end della lista.
  if (!get_edge_position(E1_ptr, el, ei1))
    return false;
    
  // controlla che E2_ptr sia associato alla relazione Vertex-Edge totale di
  // V_ptr e ne memorizza la posizione all'interno della lista (in ei2)
  // Se E2_ptr == NULL ei2 e' assegnato al past-end della lista.
  if (!get_edge_position(E2_ptr, el, ei2))
    return false;
    
  // Se il punto e' sul bordo di qualche faccia si assume che le regioni
  // adiacenti alla nuova linea creata sono quelle individuate da E1_ptr e
  // dall'ultimo spigolo spostato su V2_ptr (e2) e da E1_ptr e dall'ultimo
  // spigolo spostato su V1_ptr (e1).
  f_type * F1_ptr = NULL, * F2_ptr = NULL;
  if (!V_ptr->is_on_some_boundary()) // V_ptr non e' su qualche bordo
  {
    if (V_ptr->is_isolated())
      F1_ptr = V_ptr->containing_face();
    else
      F1_ptr = (V_ptr->incident_edge())->left_face();
  }
  else
  {
    if (E1_ptr != NULL)
      F1_ptr = E1_ptr->counter_clock_face(V_ptr);
    if (E2_ptr != NULL)
      F2_ptr = E2_ptr->counter_clock_face(V_ptr);
      
    // riassegno F1_ptr ed F2_ptr in modo che se esiste una sola faccia
    // coinvolta nell'operazione di espansione essa sia assegnata ad F1_ptr
    // e F2_ptr sia assegnato a NULL.
    if (F1_ptr == NULL)
    {
      F1_ptr = F2_ptr;
      F2_ptr = NULL;
    }
    else
      if (F1_ptr == F2_ptr)
        F2_ptr = NULL;
  }
  
  v_type * V1_ptr, *V2_ptr;
  // crea i nuovi vertici e li inserisce nella suddivisione 
  create_point(V1_ptr,V1);
  create_point(V2_ptr,V2);
  // crea la nuova line a e la inserisce nella suddivisione 
  create_line(V1_ptr,V2_ptr,E_ptr);  

  // relazione EV
  e_type * e1 = NULL, * e2 = NULL;
  if (V_ptr->is_isolated())
  {
    // modifica la relazione FV delle faccia contenente il vertice 
    F1_ptr->remove_f_vertex(V_ptr);
    F1_ptr->add_f_vertex(V1_ptr);
  }
  else  
  {
    // modifica la relazione Edge-Vertex degli spigoli compresi in senso orario
    // tra E1_ptr [E2_ptr] (rappresentato da ei1 [ei2]) ed E2_ptr [E1_ptr]
    // (rappresentato da ei2 [ei1]). Assegna ad e1 [e2] lo spigolo modificato
    // per ultimo. Se E1_ptr [E2_ptr] == NULL (ei1 [ei2] e' il past-end di el)
    // non esegue alcuna azione ed assegna e1 [e2] a NULL, altrimenti se
    // E2_ptr [E1_ptr] == NULL (ei2 [ei1] e' il past-end di el) modifica la
    // relazione di tutti gli spigoli. Se necessario modifica le relazioni FV 
    // delle facce coinvolte (che hanno nelle feature i vertici coinvolti nello
    // spostamento)
    update_edge_vertex(V_ptr, V1_ptr, ei1, ei2, el, e1);
    update_edge_vertex(V_ptr, V2_ptr, ei2, ei1, el, e2);
  }
  
  if (F2_ptr == NULL)  // E_ptr e' uno spigolo feature
  {
    // aggiorna le relazioni EE di E_ptr, E1_ptr e E2_ptr
    if (E1_ptr == NULL)
      E_ptr->assign_source_edge(E_ptr);
    else
    {
      E_ptr->assign_source_edge(e1);
      E1_ptr->assign_edge_incident_in_vertex(V1_ptr, E_ptr);
    }

    if (E2_ptr == NULL)
      E_ptr->assign_target_edge(E_ptr);
    else
    {
      E_ptr->assign_target_edge(e2);
      E2_ptr->assign_edge_incident_in_vertex(V2_ptr, E_ptr);
    }
      
    // aggiorna le relazioni EF
    E_ptr->assign_left_face(F1_ptr);
    E_ptr->assign_right_face(F1_ptr);
    
    // Nel caso in cui aggiungo una nuova feature alla regione F1_ptr devo
    // aggiornare la lista FIE
    if (!is_connect_to_feature_edges(E_ptr))
      F1_ptr->add_f_inside_edge(E_ptr);
  }
  else /* boundary */
  {
    // aggiorna le relazioni EE
    E_ptr->assign_source_edge(e1);
    E_ptr->assign_target_edge(e2);     
    E1_ptr->assign_edge_incident_in_vertex(V1_ptr, E_ptr);
    E2_ptr->assign_edge_incident_in_vertex(V2_ptr, E_ptr);

    // aggiorna le relazioni EF
    // Si assume che la regione alla destra di E_ptr sia sempre quella che si
    // e' calcolata a partire da E1_ptr (F1_ptr)
    E_ptr->assign_left_face(F2_ptr);
    E_ptr->assign_right_face(F1_ptr);    
  }

  // aggiorna le relazioni VE*
  V1_ptr->assign_incident_edge(E_ptr);
  V2_ptr->assign_incident_edge(E_ptr);
  
  // se necessario aggiorna le relazioni VF di V1_ptr e V2_ptr e la relazione
  // FV della faccia che li contiene
  if (not_on_boundary(V1_ptr))
  {
    f_type * f = E_ptr->left_face();
    V1_ptr->assign_containing_face(f);
    if (!f->is_f_vertex(V1_ptr))
      f->add_f_vertex(V1_ptr);
  }
  if (not_on_boundary(V2_ptr))
  {
    f_type * f = E_ptr->left_face();
    V2_ptr->assign_containing_face(f);
    if (!f->is_f_vertex(V2_ptr))
      f->add_f_vertex(V2_ptr);
  }
    
  // cancella il vertice vecchio dalla lista della suddivisione
  remove_entity(V_ptr);    

  pV1 = V1_ptr;
  pV2 = V2_ptr;
    
  return true;
}

// *************** LINE TO POINT CONTRACTION **********************************
// Contrae lo spigolo indicato da E_ptr nel punto V e assegna V_ptr a
// rappresentare il nuovo punto. E_ptr non puo' essere un cappio.
// Inoltre se E_ptr appartiene ad una faccia con due soli spigoli, tale faccia
// non deve avere feature interne (spigoli o buchi) con un vertici in comune
// con E_ptr (questo per non creare cappi con tali feature interne).
// In caso di errore V_ptr viene assegnato a NULL.

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
line_to_point_contraction(e_type *& E_ptr, v_type V, v_type *& V_ptr)
{
  if (E_ptr == NULL)
    return false;
  if (E_ptr->is_closed())
    return false;
  
  // memorizza la realzione edge-edge di E_ptr  
  e_type * e00, * e01, * e10, * e11;
  edge_edge(E_ptr, e00, e01, e10, e11); 
  
  // controllo che se E_ptr appartiene ad una faccia con due soli spigoli,
  // tale faccia non abbia feature (spigoli o buchi) connesse a E_ptr
  f_type * left_f = NULL, * right_f = NULL;
  if (!E_ptr->is_in_feature())
  {
    e_list_type el;
    f_type * f = E_ptr->left_face();
    if (!is_infinity_face(f))
    {
      get_boundary_edges(f, E_ptr, el);
      if (el.size() == 2)
      {
        left_f = f; 
        if ((!el.is_in_list(e00)) || (!el.is_in_list(e10)))
          return false;
      }
    }
    f = E_ptr->right_face();
    if (!is_infinity_face(f))
    {
      el.free();
      get_boundary_edges(f, E_ptr, el);
      if (el.size() == 2)
      {
        right_f = f;
        if ((!el.is_in_list(e01)) || (!el.is_in_list(e11)))
          return false;
      }
    }
  }

  v_type * v1 = E_ptr->source_vertex(), * v2 = E_ptr->target_vertex();
  
  // memorizza rel reazioni EE di e00 in v1 e di e11 in v2
  e_type * e00cc = e00->edge_incident_in_vertex(v1);
  e_type * e11cc = e11->edge_incident_in_vertex(v2);
  
  if (E_ptr->is_in_feature())
  {
    // nel caso E_ptr fosse il rappresentante di una feature di f
    // lo sostituisce nella relazione FIE
    f_type * f = E_ptr->left_face();
    ei_type E_pos;
    if (f->is_f_inside_edge(E_ptr, E_pos))
    {
      // e0 ed e1 rispettivamente gli spigoli incidenti nel vertice source e
      // nel vertice target di E_ptr, ad esso successivi (in senso antiorario)
      // e appartenenti alla medesima feature
      e_type * e0, * e1;
      if (!get_next_feature_edges(E_ptr, e0, e1))
        return false;
      f->remove_f_inside_edge(E_pos);
      if (E_ptr != e0)
        f->add_f_inside_edge(e0);
      else if (E_ptr != e1)
        f->add_f_inside_edge(e1);
    }
    
    // Elimina i vertici di E_ptr dalla relazione FV di f
    vi_type v_pos;
    if (f->is_f_vertex(v1, v_pos))
      f->remove_f_vertex(v_pos);
    if (f->is_f_vertex(v2, v_pos))
      f->remove_f_vertex(v_pos);
  }
  else   // caso boundary
  {
    // nel caso E_ptr fosse il rappresentante del contorno (esterno o 
    // interno di qualche faccia lo sostituisce nelle relazionni)
    f_type * f = E_ptr->left_face();
    e_type * new_e = E_ptr->next_clock_edge(f);
    ei_type ei;
    if (E_ptr == f->bounding_edge())
      f->assign_bounding_edge(new_e);
    if (f->is_inside_edge(E_ptr, ei))
    {
      f->remove_inside_edge(ei);
      f->add_inside_edge(new_e);
    }
    f = E_ptr->right_face();
    new_e = E_ptr->next_clock_edge(f);
    if (E_ptr == f->bounding_edge())
      f->assign_bounding_edge(new_e);
    if (f->is_inside_edge(E_ptr, ei))
    {
      f->remove_inside_edge(ei);
      f->add_inside_edge(new_e);
    }
  }

  create_point(V_ptr,V);
    
  // aggiorna le relazioni EV degli spigoli incidenti in V1 e V2
  e_type * ek = E_ptr->source_edge();
  while (ek != E_ptr)
  {
    if (ek->is_closed())
    {
      ek->assign_source_vertex(V_ptr);
      ek->assign_target_vertex(V_ptr);
      ek = ek->source_edge();
    }
    else if (ek->source_vertex() == v1)
    {
      ek->assign_source_vertex(V_ptr);
      ek = ek->source_edge();
    }
    else
    {
      ek->assign_target_vertex(V_ptr);
      ek = ek->target_edge();
    }
  }
  
  ek = E_ptr->target_edge();
  while (ek != E_ptr)
  {
    if (ek->is_closed())
    {
      ek->assign_source_vertex(V_ptr);
      ek->assign_target_vertex(V_ptr);
      ek = ek->source_edge();
    }
    else if (ek->source_vertex() == v2)
    {
      ek->assign_source_vertex(V_ptr);
      ek = ek->source_edge();
    }
    else
    {
      ek->assign_target_vertex(V_ptr);
      ek = ek->target_edge();
    }
  }
  
  // aggiorna la relazione  VE* del nuovo vertice V
  if (e00 != E_ptr)
    V_ptr->assign_incident_edge(e00);
  else if (e11 != E_ptr)
    V_ptr->assign_incident_edge(e11);
  else // in questo caso E_ptr era uno spigolo isolato
  {
    f_type * f = E_ptr->left_face();
    V_ptr->assign_containing_face(f);
    V_ptr->assign_incident_edge(NULL);
  }

  if (left_f != NULL) 
  {
    // se left_f e' stata assegnata rappresenta una faccia con due soli 
    // spigoli (che diverra' un cappio orientato in senso antiorario)
    if (e00cc == E_ptr)
      e00->assign_source_edge(e11);
    else
    {
      e00->assign_source_edge(e00cc);
      e01->assign_edge_incident_in_vertex(V_ptr, e11);
    }
    e00->assign_target_edge(e00);
    
    // assegna al cappio un orientamento antiorario
    e00->assign_right_face(e00->other_face(left_f));
    e00->assign_left_face(left_f);
  }
  if (right_f != NULL)
  {
    // se right_f e' stata assegnata rappresenta una faccia con due soli 
    // spigoli (che diverra' un cappio orientato in senso antiorario)
    if (e11cc == E_ptr)
      e11->assign_source_edge(e00);
    else
      e11->assign_source_edge(e10);
    {
      e11->assign_source_edge(e11cc);
      e10->assign_edge_incident_in_vertex(V_ptr, e00);
    }
    
    // assegna al cappio un orientamento antiorario
    e11->assign_right_face(e11->other_face(right_f));
    e11->assign_left_face(right_f);
  }
  
  if ((left_f == NULL) && (right_f == NULL))
  {
    // aggiorna le relazioni EE degli edge coinvolti
    if (e11 != E_ptr)
      e01->assign_edge_incident_in_vertex(V_ptr, e11);
    else
      e01->assign_edge_incident_in_vertex(V_ptr, e00);
    if (e00 != E_ptr)
      e10->assign_edge_incident_in_vertex(V_ptr, e00);
    else
      e10->assign_edge_incident_in_vertex(V_ptr, e11);
  }
  
  // assegna se necessario la relazione FV della faccia contenente il nuovo
  // vertice
  if (!(v1->is_on_some_boundary()) && (!v2->is_on_some_boundary()))
  {
    f_type * f = E_ptr->left_face();
    f->add_f_vertex(V_ptr);
  }
  
  // cancella i vertici vecchi dalla lista della suddivisione
  remove_entity(v1);    
  remove_entity(v2);    
  
  // cancella l'edge dalla lista della suddivisione
  remove_entity(E_ptr);    
  
  return true;
}


// *************** EXPAND POINT TO REGION *************************************
// Espande un vertice in un cappio orientato in senso antiorario.
// - V_ptr e' il vertice da espandere.
// - E1_ptr e' lo spigolo che determina la posizione del cappio. Esso
//   viene infatti inserito nel settore delimitato da E1_ptr e dallo spigolo
//   associato alla relazione Edge-Edge di E1_ptr applicata nel vertice
//   V_ptr.
// - F1_ptr e' la faccia in cui inserire il cappio.
// Condizioni per la creazione di un cappio sono che il cappio sia isolato
// (V_ptr vertice isolato) e E1_ptr == NULL, oppure che E1_ptr appartenga al
// contorno o ad una feature di F1_ptr e sia incidente in V_ptr.
// Il vertice rappresentante il cappio e' assegnato a E_ptr. Le sue relazioni
// Edge-Edge sono assegnate entrambe al vertice associato alla relazione
// Edge-Edge di E1_ptr applicata nel vertice V_ptr.
// Lo spigolo e la faccia creati vengono assegnati rispettivamente a E_ptr e
// F_ptr. In caso di errore E_ptr e F_ptr sono assegnati a NULL.

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
expand_point_to_region(v_type * V_ptr, e_type * E1_ptr, f_type * F1_ptr,
		       e_type *& E_ptr, f_type *& F_ptr)
{
  E_ptr = NULL;
  F_ptr = NULL;

  if ((V_ptr == NULL) || (F1_ptr == NULL))
    return false;
  
  // non si possono inserire feature (spigoli o buchi) centro un cappio
  // che abbiano un vertice in comune con esso
  if (!is_infinity_face(F1_ptr))
  {
    e_type * e = F1_ptr->bounding_edge();
    if ((e->is_closed()) && (e->source_vertex() == V_ptr))
      return false;
  }
    
  // nel caso di un cappio non isolato (E1_ptr != NULL) controllo che
  // il vertice non sia isolato e che lo spigolo appartenga alla regione
  // (sia sul bordo o in una feature) e sia incidente nel vertice
  if (E1_ptr != NULL)	// cappio non isolato
  {
    if ((!(E1_ptr->is_incident(V_ptr))) ||
        (!(edge_is_region_component(E1_ptr, F1_ptr))))
      return false;
  }
  else
  {
    if (!V_ptr->is_isolated())
      return false;
    if (V_ptr->containing_face() != F1_ptr)
      return false;
  }
  
  create_line(V_ptr,V_ptr,E_ptr);
  create_face(E_ptr,F_ptr);

  // relazione VF
  V_ptr->assign_containing_face(NULL);

  e_type * aux_e;
  
  // se necessario aggiorno la relazione IE di F1_ptr
  if (!is_on_some_boundary(V_ptr))
    F1_ptr->add_inside_edge(E_ptr);

  if (E1_ptr != NULL)
  {
    // vertice associato alla relazione EE e incidente in V_ptr
    aux_e = E1_ptr->edge_incident_in_vertex(V_ptr);
    E_ptr->assign_source_edge(aux_e);
    E_ptr->assign_target_edge(E_ptr);

    // aggiorno le relazioni EE di E1_ptr 
    E1_ptr->assign_edge_incident_in_vertex(V_ptr, E_ptr);
  }
  else
  {
    // aggiorno la relazione VE* ed EE
    E_ptr->assign_source_edge(E_ptr);
    E_ptr->assign_target_edge(E_ptr);    
    V_ptr->assign_incident_edge(E_ptr);
  }
  
  // aggiorno la relazione EF
  E_ptr->assign_left_face(F_ptr);
  E_ptr->assign_right_face(F1_ptr);

  // aggiorno la relazione FE*
  F_ptr->assign_bounding_edge(E_ptr);
    
  // aggiorno la relazione FV
  vi_type V_pos;
  if (F1_ptr->is_f_vertex(V_ptr,V_pos))
    F1_ptr->remove_f_vertex(V_pos);
      
  return true;
}


// *************** REGION TO POINT CONTRACTION ********************************
// Contrae un cappio in un punto. Si suppone che entrambe le sue relazioni EE
// siano assegnate allo spigolo che si incontra ruotando in senso antiorario.
// Condizioni per la contrazione di un cappio sono che esso non contenga buchi
// o features. In caso di successo reinizializza F_ptr a NULL.
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
region_to_point_contraction(f_type *& F_ptr)
{
  e_type * E_ptr;
  v_type * V_ptr;
  f_type * F1_ptr;
  
  if (F_ptr == NULL)
    return false;
  if (is_infinity_face(F_ptr))
    return false;
    
  E_ptr = F_ptr->bounding_edge();
  
  if ((!(E_ptr->is_closed())) ||
      (!(F_ptr->no_f_vertices())) || 
      (!(F_ptr->no_f_inside_edges())) || 
      (!(F_ptr->no_inside_edges())))
    return false;
   
  V_ptr = E_ptr->source_vertex();
  F1_ptr = E_ptr->right_face();

  // aggiornamento relazioni
  if (!E_ptr->is_isolated())	// cappio non isolato
  {
    // relazione IE
    // nel caso E_ptr fosse stato il rappresentante di un buco aggiorna la 
    // la relazione IE di F1_ptr
    ei_type ei;
    if (F1_ptr->is_inside_edge(E_ptr, ei))
    {
      F1_ptr->remove_inside_edge(ei);
      e_type * e;
      if (get_next_boundary_edge(V_ptr, E_ptr, e))
        F1_ptr->add_inside_edge(e);
    }
      
    // aggiorno VE*
    if (V_ptr->incident_edge() == E_ptr)
      V_ptr->assign_incident_edge(E_ptr->edge_incident_in_vertex(V_ptr));
      
    // aggiorno EE
    e_type * e = E_ptr->source_edge();
    e_type * e1 = e;
    while (e1->edge_incident_in_vertex(V_ptr) != E_ptr)
      e1 = e1->edge_incident_in_vertex(V_ptr);
    e1->assign_edge_incident_in_vertex(V_ptr, e);
  }
  else	// cappio isolato
  {
    // elimina E_ptr dalla relazione IE di F1_ptr
    F1_ptr->remove_inside_edge(E_ptr);
    V_ptr->assign_incident_edge(NULL);
  }

  // se necessario aggiorna le relazioni VF di V_ptr e FV della faccia che lo
  // contiene
  if (not_on_boundary(V_ptr))
  {
    V_ptr->assign_containing_face(F1_ptr);
    F1_ptr->add_f_vertex(V_ptr);
  }
 
  // elimina l'edge dalla dalla suddivisione
  remove_entity(E_ptr);    
  // elimina la faccia dalla suddivisione
  remove_entity(F_ptr);    

  return true;
}


// *************** EXPAND LINE TO REGION **************************************
// Espande lo spigolo E_ptr nella regione F_ptr. La regione e' formata da
// due spigoli che hanno entrambi come vertici i vertici di E_ptr.
// L'algoritmo riutilizza E_ptr come lo spigolo che ha alla propria
// sinistra la regione creata.
// Condizioni necessarie all'espansione sono che lo spigolo non rappresenti
// un cappio.

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
expand_line_to_region(e_type * E_ptr, f_type *& F_ptr, e_type *& E2_ptr)
{
  F_ptr = NULL;
  E2_ptr = NULL;

  if (E_ptr == NULL)
    return false;
    
  if (E_ptr->is_closed())
    return false;
        
  v_type * V1_ptr = E_ptr->source_vertex();
  v_type * V2_ptr = E_ptr->target_vertex();

  // memorizzo la relazione EE di E_ptr
  e_type * e00, * e01, * e10, * e11;
  edge_edge(E_ptr, e00, e01, e10, e11);
  
  // nel caso E_ptr sia uno spigolo feature memorizzo se e' in contatto con il
  // bordo di qualche faccia
  bool touch_bound;
  if (E_ptr->is_in_feature())
    touch_bound = is_on_some_boundary(V1_ptr) || is_on_some_boundary(V2_ptr);
  
  // nel caso E_ptr fosse una feature, memorizza in  es ed et rispettivamente
  // gli spigoli incidenti nel vertice source e nel vertice target di E_ptr,
  // ad esso successivi (in senso antiorario) e appartenenti alla medesima
  // feature
  e_type * es, * et;
  if (E_ptr->is_in_feature()) 
  {
    if (!get_next_feature_edges(E_ptr, es, et))
      return false;
  }

  // per evitare interferenze causate dal riutilizzo di E_ptr creo una copia
  // locale dello spigolo indicato E_ptr
  e_type loc_E = * E_ptr;
  e_type * loc_E_ptr = & loc_E;
  
  // creo il secondo spigolo
  create_line(V1_ptr,V2_ptr,E2_ptr);
  
  // creo la regione
  create_face(E_ptr,F_ptr);
  
  // relazione EF dei nuovi spigoli
  E_ptr->assign_left_face(F_ptr);
  E2_ptr->assign_left_face(loc_E_ptr->left_face());
  E2_ptr->assign_right_face(F_ptr);

  // relazione EE* dei nuovi spigoli e degli spigoli in essi incidenti
  E_ptr->assign_source_edge(E2_ptr);
  E2_ptr->assign_source_edge(e00);
  E2_ptr->assign_target_edge(E_ptr);
  if ((e10->source_vertex() == loc_E_ptr->source_vertex()) ||
      (e10->target_vertex() == loc_E_ptr->source_vertex()))
  {
    e10->assign_target_edge(E2_ptr);
  }
  else
  {
    if (e10->source_vertex() == loc_E_ptr->target_vertex())
      e10->assign_source_edge(E2_ptr);
    else
      e10->assign_target_edge(E2_ptr);
  }
  
  f_type * left_f = loc_E_ptr->left_face();
  if (loc_E_ptr->is_in_feature()) // E_ptr era una feature
  {
    // aggiorna la relazione FIE della faccia che conteneva E_ptr
    update_feature_rel(left_f, E_ptr, es, et);
    
    // aggiorno la relazione FV della faccia che conteneva E_ptr
    vi_type vi;
    if (left_f->is_f_vertex(V1_ptr, vi))
      left_f->remove_f_vertex(vi);
    if (left_f->is_f_vertex(V2_ptr, vi))
      left_f->remove_f_vertex(vi);
      
    // aggiorna le relazioni VF di V1_ptr e V2_ptr
    V1_ptr->assign_containing_face(NULL);
    V2_ptr->assign_containing_face(NULL);
    
    // se necessario aggiorna la relazione IE della faccia che contiene quella
    // creata
    if (!touch_bound)
      left_f->add_inside_edge(E_ptr);
  }
  else // E_ptr era sul bordo di una faccia
  {
    // aggiorna la relazione FE* di left_f
    // (E_ptr e' diventato rappresentante della nuova regione,
    // nel caso fosse stato rappresentante di left_f ad essa ne viene
    // assegnato uno nuovo)
    if (left_f->bounding_edge() == E_ptr)
      left_f->assign_bounding_edge(e00);
      
    // se necessario aggiorna la relazione IE della faccia che contiene quella
    // creata (essa deve essere una delle due associate alla vecchia relazione
    // EF di E_ptr)
    f_type * f = loc_E_ptr->left_face();
    ei_type ei;
    if (!E_ptr->is_bounding(f)) // E_ptr non e' piu' di contorno
      if (f->is_inside_edge(E_ptr, ei))
      {
        f->remove_inside_edge(ei);
        f->add_inside_edge(E2_ptr);
      }
    f = loc_E_ptr->right_face();
    if (!E_ptr->is_bounding(f)) // E_ptr non e' piu' di contorno
      if (f->is_inside_edge(E_ptr, ei))
      {
        f->remove_inside_edge(ei);
        f->add_inside_edge(E2_ptr);
      }
  }

  return true;
}

// *************** REGION TO LINE CONTRACTION *********************************
// Contrae la regione indicata da F_ptr in uno spigolo e assegna E_ptr a tale
// spigolo. La regione deve essere composta da due soli spigoli e non deve
// avere buchi o features. In caso di successo reinizializza F_ptr a NULL.
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
region_to_line_contraction(f_type *& F_ptr, e_type *& E_ptr)
{
  if (F_ptr == NULL)
    return false;
  if ((!F_ptr->no_f_vertices()) || (!F_ptr->no_f_inside_edges()) ||
      (!F_ptr->no_inside_edges()))
    return false;
  e_type * e0 = F_ptr->bounding_edge();
  if (e0->is_closed()) // cappio
    return false;
  
  v_type * v0 = e0->source_vertex(), * v1 = e0->target_vertex();
  e_type * e1 = e0->next_clock_edge(F_ptr);
 
  if (e1->next_clock_edge(F_ptr) != e0) // regione con piu' spigoli
    return false;
 
  E_ptr = e0;
  // per evitare interferenze causate dal riutilizzo di E_ptr creo una copia
  // locale dello spigolo indicato E_ptr
  e_type loc_E = * E_ptr;
  e_type * loc_E_ptr = & loc_E;
  
  // assegno la relazione EF di E_ptr
  f_type * f = e1->other_face(F_ptr);
  
  if (loc_E_ptr->left_face() == F_ptr)
    E_ptr->assign_left_face(f);
  else
    E_ptr->assign_right_face(f);
 
  // aggiorno la relazione EE degli spigoli adiacenti a E_ptr
  v_type * vk = v1;
  e_type * ek = e1->edge_incident_in_vertex(vk);
  
  if (ek != E_ptr)
  {
    vk = v0;
    ek = e1->edge_incident_in_vertex(vk);
  }
 
  while (ek->edge_incident_in_vertex(vk) != e1)
  {    
    ek = ek->edge_incident_in_vertex(vk);
  }
  if (ek->source_vertex() == vk)
    ek->assign_source_edge(E_ptr);
  else
    ek->assign_target_edge(E_ptr);

  // assegno le relazioni EE di E_ptr
  if (E_ptr->source_edge() == e1)
  {
    if (e1->source_vertex() == v0)
      E_ptr->assign_source_edge(e1->source_edge());
    else
      E_ptr->assign_source_edge(e1->target_edge());
  }
  if (E_ptr->target_edge() == e1)
  {
    if (e1->source_vertex() == v0)
      E_ptr->assign_target_edge(e1->target_edge());
    else
      E_ptr->assign_target_edge(e1->source_edge());
  }

  // nel caso e1 fosse stato il rappresentante del bordo della faccia f
  // ne aggiorno la relazione FE
  if (f->bounding_edge() == e1)
    f->assign_bounding_edge(E_ptr);
    
  // se necessario aggiorno la relazione IE del padre di F_ptr (deve essere
  // una delle due facce associate alla relazione EE di E_ptr)
  f = E_ptr->left_face();
  ei_type ei;
  e_type * e;
  if (f->is_inside_edge(e1, ei))
  {
    f->remove_inside_edge(ei);
    if (get_next_boundary_edge(v0, E_ptr, e))
      f->add_inside_edge(e);
    else
      if (get_next_boundary_edge(v1, E_ptr, e))
        f->add_inside_edge(e);
  }
  if (E_ptr->is_in_feature())
   if (f->is_inside_edge(E_ptr, ei))
   {
     f->remove_inside_edge(ei);
     if (get_next_boundary_edge(v0, E_ptr, e))
       f->add_inside_edge(e);
     else
       if (get_next_boundary_edge(v1, E_ptr, e))
         f->add_inside_edge(e);
  }
  
  f = E_ptr->right_face();
  if (f->is_inside_edge(e1, ei))
  {
    f->remove_inside_edge(ei);
    if (get_next_boundary_edge(v0, E_ptr, e))
      f->add_inside_edge(e);
    else
      if (get_next_boundary_edge(v1, E_ptr, e))
        f->add_inside_edge(e);
  }
  if (E_ptr->is_in_feature())
   if (f->is_inside_edge(E_ptr, ei))
   {
     f->remove_inside_edge(ei);
     if (get_next_boundary_edge(v0, E_ptr, e))
       f->add_inside_edge(e);
     else
       if (get_next_boundary_edge(v1, E_ptr, e))
         f->add_inside_edge(e);
  }
    
  // nel caso che ora E_ptr appartenga ad una feature aggiorno la relazione
  // FIE della faccia contenente la feature.
  if (E_ptr->is_in_feature())
  {
    f_type * f = E_ptr->left_face();
    bool e_found = false;
    ei_type ei_end = f->f_inside_edges_end();
    for (ei_type ei = f->f_inside_edges_begin(); ei != ei_end; ei++)
    {
      e_type * e = * ei;    
      e_list_type el;
      feature_edges(e,el);
      if (el.is_in_list(E_ptr))
      {
        if (e_found) // E_ptr collega due feature
        {
          f->remove_f_inside_edge(ei);
          break;
        }
        else
          e_found = true;
      }
    }
    if (!e_found)  // e' stata creata una nuova feature
      f->add_f_inside_edge(E_ptr);
      
    // se necessario aggiorna le relazioni FV di f e VF di v0 e v1
    if (not_on_boundary(v0))
    {
      v0->assign_containing_face(f);
      f->add_f_vertex(v0);
    }
    if (not_on_boundary(v1))
    {
      v1->assign_containing_face(f);
      f->add_f_vertex(v1);
    }
  }
    
  // elimina l'edge dalla dalla suddivisione
  remove_entity(e1);    
  // elimina la faccia dalla suddivisione
  remove_entity(F_ptr);
  
  return true; 
}


// *************** SPLIT LINE *************************************************
// Divide in due lo spigolo indicato da E0_ptr creando il nuovo spigolo E_ptr.
// Il vertice V0 indica il punto in cui dividere lo spigolo, V_ptr e' il
// nuovo punto creato. In pratica se E0_ptr = (v1,v2), si ottengono i due
// nuovi spigoli E0_ptr = (v1,v0) ed E_ptr = (v0,v2);
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
split_line(e_type * E0_ptr, v_type V0, v_type  *& V_ptr, e_type *& E_ptr)
{
  V_ptr = NULL;
  E_ptr = NULL;

  if (E0_ptr == NULL)
    return false;

  v_type * v1 = E0_ptr->source_vertex(), * v2 = E0_ptr->target_vertex();
  // creazione delle nuove entita'
  create_point(V_ptr,V0);
  create_line(V_ptr,v2,E_ptr);

  // relazione VE di V_ptr
  V_ptr->assign_incident_edge(E0_ptr);
  
  // relazione EE degli spigoli incidenti in E0_ptr;
  e_type * ek = E0_ptr->edge_incident_in_vertex(v2), * e1 = NULL;
  while (ek != E0_ptr)
  {    
    e1 = ek;
    ek = ek->edge_incident_in_vertex(v2);
  }
  if (e1 != NULL)
    e1->assign_edge_incident_in_vertex(v2, E_ptr);
  
  // relazione EE di E_ptr
  E_ptr->assign_source_edge(E0_ptr);
  if ((E0_ptr->target_edge() == E0_ptr) && (!E0_ptr->is_closed()))
    E_ptr->assign_target_edge(E_ptr);
  else
  {
    if (E0_ptr->is_closed())
      E_ptr->assign_target_edge(E0_ptr->source_edge());
    else
     E_ptr->assign_target_edge(E0_ptr->target_edge());
  }
  
  // relazione EF di E_ptr
  E_ptr->assign_left_face(E0_ptr->left_face());
  E_ptr->assign_right_face(E0_ptr->right_face());

  // relazione EV di E0_ptr  
  E0_ptr->assign_target_vertex(V_ptr);
  
  // relazione EE di E0_ptr  
  E0_ptr->assign_target_edge(E_ptr);
  if (E_ptr->target_vertex() == E0_ptr->source_vertex())
    E0_ptr->assign_source_edge(E_ptr); // E0_ptr era un cappio
    
  // relazione VF di V_ptr e FV della faccia che lo contiene
  if (!v1->is_on_some_boundary())
  {
    f_type * f = v1->containing_face();
    V_ptr->assign_containing_face(f);
    f->add_f_vertex(V_ptr);
  }
  
  return true;
}


// *************** LINE MERGE *************************************************
// connette i due spigoli incidenti nel vertice indicato da V0_ptr. E_ptr
// rappresenta lo spigolo ottenuto dalla connessione. Condizione necessaria
// alla funzione e' che in V0_ptr siano incidenti solo due spigoli. In caso
// di successo reinizializza V0_ptr a NULL;
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
line_merge(v_type *& V0_ptr, e_type *& E_ptr)
{
  if (V0_ptr == NULL)
    return false;
  if (V0_ptr->is_isolated())
    return false;
  e_type * e1 = V0_ptr->incident_edge();
  e_type * e2 = e1->edge_incident_in_vertex(V0_ptr);
  if (e2->edge_incident_in_vertex(V0_ptr) != e1)
    return false;
    
  // riutilizzo e1
  E_ptr = e1;
  // per evitare interferenze causate dal riutilizzo di E_ptr creo una copia
  // locale dello spigolo indicato E_ptr
  e_type loc_E = * E_ptr;
  e_type * loc_E_ptr = & loc_E;
  
  // relazione EV
  v_type * v1 = loc_E_ptr->other_vertex(V0_ptr);
  v_type * v2 = e2->other_vertex(V0_ptr);
  E_ptr->assign_target_vertex(v2);
  
  if (v1 == v2)
  {
    // relazione EE degli spigoli incidenti in e2;
    e_type * ek = e2->edge_incident_in_vertex(v2), * ec = NULL;
    while (ek != e2)
    {    
      ec = ek;
      ek = ek->edge_incident_in_vertex(v2);
    }
    if ((ec != NULL) && (ec != E_ptr))
      ec->assign_edge_incident_in_vertex(v2, E_ptr);
    
    // relazione EE di E_ptr
    e_type * e0 = e2->edge_incident_in_vertex(v2);
    if (e0 == e1)
      e0 = e1->edge_incident_in_vertex(v2);
      if (e0 == e2)
        e0 = e1;
    E_ptr->assign_source_edge(e0);
    E_ptr->assign_target_edge(e0);
  }
  else
  {
    // relazione EE degli spigoli incidenti in e2;
    e_type * ek = e2->edge_incident_in_vertex(v2), * ec = NULL;
    while (ek != e2)
    {    
      ec = ek;
      ek = ek->edge_incident_in_vertex(v2);
    }
    if (ec != NULL)
      ec->assign_edge_incident_in_vertex(v2, E_ptr);
    
    // relazione EE di E_ptr
    e_type * e0 = e2->edge_incident_in_vertex(v2);
    if (e0 == e2)
      e0 = E_ptr;
    E_ptr->assign_edge_incident_in_vertex(V0_ptr, e0);
  }
  
  if (!loc_E_ptr->is_in_feature())  // boundary
  {
    // nel caso e2 fosse rappresentante lo sostituisce nella relazione FE
    // delle facce coinvolte
    f_type * f = loc_E_ptr->left_face();
    if (f->bounding_edge() == e2)
      f->assign_bounding_edge(E_ptr);
    f = loc_E_ptr->right_face();
    if (f->bounding_edge() == e2)
      f->assign_bounding_edge(E_ptr);
   }
   else // feature
   {
     ei_type ei;
     // nel caso e2 fosse rappresentante lo sostituisce nella relazione FIE
     // delle facce coinvolte
     f_type * f = loc_E_ptr->left_face();
     if (f->is_f_inside_edge(e2,ei))
     {
       f->remove_f_inside_edge(ei);
       f->add_f_inside_edge(E_ptr);
     }
     
     // Elimina V0_ptr dalla relazione FV di f
     f->remove_f_vertex(V0_ptr);
   }
   
  // elimina il vertice dalla suddivisione
  remove_entity(V0_ptr);    
  // elimina l'edge dalla dalla suddivisione
  remove_entity(e2);    
  
  return true;
}


// *************** SPLIT REGION ***********************************************
// Divide in due la regione indicata da F0_ptr, unendo i punti V1_ptr e V2_ptr
// creando lo spigolo E_ptr e la nuova regione F_ptr.
// Eventuali feature e buchi rimangono comunque assegnati alla regione
// rappresentata da F0_ptr. Tale regione e' quella alla sinistra dello spigolo
// orientato di vertici V1_ptr e V2_ptr
template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
split_region(f_type * F0_ptr,  v_type * V1_ptr, v_type * V2_ptr,
	     e_type  *& E_ptr, f_type *& F_ptr)
{
  if ((F0_ptr == NULL) || (V1_ptr == NULL) || (V2_ptr == NULL))
    return false;
  if (V1_ptr == V2_ptr)
    return false;
  if ((V1_ptr->is_isolated()) || (V2_ptr->is_isolated()))
    return false;
   
  e_list_type path1;
  v_type * v1_bound;
  e_type * e11_bound, * e12_bound;
  if (!path_to_boundary(V1_ptr, F0_ptr, path1, v1_bound, e11_bound, e12_bound))
      return false;

  e_list_type path2;
  v_type * v2_bound;
  e_type * e21_bound, * e22_bound;
  if (!path_to_boundary(V2_ptr, F0_ptr, path2, v2_bound, e21_bound, e22_bound))
      return false;
      
  create_line(V1_ptr, V2_ptr, E_ptr);
  create_face(E_ptr, F_ptr);
    
  // relazione EE di E_ptr e degli spigoli in esso incidenti
  if (path1.empty()) // V1_ptr e' sul boundary
  {
    E_ptr->assign_source_edge(e12_bound);
    e11_bound->assign_edge_incident_in_vertex(V1_ptr, E_ptr);
  }
  else // V1_ptr e' su una feature
  {
    e_type * ek;
    get_edge_incident_in_vertex(V1_ptr, path1, ek);
    
    // se necessario elimino V1_ptr dalla relazione FV di F0_ptr
    if (ek->edge_incident_in_vertex(V1_ptr) == ek)
      F0_ptr->remove_f_vertex(V1_ptr);
    E_ptr->assign_source_edge(ek);
    ek->assign_edge_incident_in_vertex(V1_ptr, E_ptr);
  }
  if (path2.empty()) // V2_ptr e' sul boundary
  {
    E_ptr->assign_target_edge(e22_bound);
    e21_bound->assign_edge_incident_in_vertex(V2_ptr, E_ptr);
  }
  else // V2_ptr e' su una feature
  {
    e_type * ek;
    get_edge_incident_in_vertex(V2_ptr, path2, ek);
    
    // se necessario elimino V2_ptr dalla relazione FIE di F0_ptr
    if (ek->edge_incident_in_vertex(V2_ptr) == ek)
      F0_ptr->remove_f_vertex(V2_ptr);
    
    E_ptr->assign_target_edge(ek);
    ek->assign_edge_incident_in_vertex(V2_ptr, E_ptr);
  }
  
  // aggiornamento della relazione EF degli spigoli in comune tra le due
  // regioni, della relazione FV dei loro vertici e della  relazione FIE di
  // F0_ptr
  E_ptr->assign_left_face(F0_ptr);
  E_ptr->assign_right_face(F_ptr);
  e_type * ek;
  v_type * vk = v1_bound;
  while (vk != V1_ptr)
  {
    get_edge_incident_in_vertex(vk, path1, ek);
    if (ek->source_vertex() == vk)
      ek->assign_right_face(F_ptr);
    else
      ek->assign_left_face(F_ptr);
    F0_ptr->remove_f_inside_edge(ek);
    vk = ek->other_vertex(vk);
    vk->assign_containing_face(NULL);
  }
  vk = v2_bound;
  while (vk != V2_ptr)
  {
    get_edge_incident_in_vertex(vk, path2, ek);
    if (ek->source_vertex() == vk)
      ek->assign_left_face(F_ptr);
    else
      ek->assign_right_face(F_ptr);
    F0_ptr->remove_f_inside_edge(ek);
    vk = ek->other_vertex(vk);
    vk->assign_containing_face(NULL);
  }
  
  // relazione EF deggli spigoli non in comune tra le due regioni
  e_type * e0;
  ek = e11_bound;
  do
  {
    e0 = ek;
    ek = ek->next_clock_edge(F0_ptr);
    if (e0->left_face() == F0_ptr)
      e0->assign_left_face(F_ptr);
    else
      e0->assign_right_face(F_ptr);
  }
  while (e0 != e22_bound);
  
  // nel caso lo spigolo associato alla relazione BE di F0_ptr non sia piu'
  // sul suo boundari lo sostituisce con E_ptr
  if (!(F0_ptr->bounding_edge()->is_bounding(F0_ptr)))
    F0_ptr->assign_bounding_edge(E_ptr);
      
  return true;  
}

// *************** REGION MERGE ***********************************************
// crea la regione F_ptr unendo le due regioni cui lo spigolo E0_ptr
// appartiene. L'unione avviene eliminando lo spigolo da boundary ed assegnando
// come features di F_ptr eventuali altri spigoli appartenenti al bordo comune
// alle due regioni iniziali. In caso di successo reinizializza E0_ptr a NULL.

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::
region_merge(e_type *& E0_ptr, f_type *& F_ptr)
{
  if (E0_ptr == NULL)
    return false;

  if (E0_ptr->is_in_feature())
    return false;

  f_type * f_dest = E0_ptr->left_face(), * f2 = E0_ptr->right_face();
  f_type * f0;

  // f_dest sara' la faccia riutilizzata per rappresntare F_ptr. Si controlla
  // che, nel caso, non sia interna a f2. Questo, perche' nel caso
  // una faccia sia interna all'altra, il contorno della faccia interna
  // diverra' una feature della faccia esterna
  if (is_inside_face(f_dest, f2))
  {
    f0 = f_dest;
    f_dest = f2;
    f2 = f0;
  }
     
  // assegna ad el la lista degli spigoli appartenenti al bordo comune (E0_ptr
  // compreso)
  e_list_type el;
  get_connected_common_edges(E0_ptr, f_dest, f2, el);
  
  // controlla che la lista non sia vuota, sia connessa e assegna ad e1 ed e2
  // gli spigoli piu' esterni della lista (quelli in contatto con i bordi non
  // in comune delle due facce)
  e_type * e1, *e2;
  if (!get_external_edges(el, e1, e2))
    return false;

  // assegna ad f_dest (riutilizzata come nuova faccia) le features e i
  // buchi di f2 (eventuali duplicazioni vengono gestite alla fine)
  if (!f2->no_f_vertices())
  {
    v_type * vk;
    vi_type vi_end = f2->f_vertices_end();
    for (vi_type vi = f2->f_vertices_begin(); vi != vi_end; vi++)
    {
      vk = * vi;
      vk->assign_containing_face(f_dest);
      f_dest->add_f_vertex(vk);
    }
  }

  if (!f2->no_f_inside_edges())
  {
    e_type * ek;
    ei_type ei_end = f2->f_inside_edges_end();
    for (ei_type ei = f2->f_inside_edges_begin(); ei != ei_end; ei++)
    {
      ek = * ei;
      ek->assign_left_face(f_dest);
      ek->assign_right_face(f_dest);
      f_dest->add_f_inside_edge(* ei);
    }
  }

  if (!f2->no_inside_edges())
  {
    e_type * ek;
    ei_type ei_end = f2->inside_edges_end();
    for (ei_type ei = f2->inside_edges_begin(); ei != ei_end; ei++)
    {
      ek = * ei;
      if (ek->left_face() == f2)
        ek->assign_left_face(f_dest);
      else
        ek->assign_right_face(f_dest);
      f_dest->add_inside_edge(ek);
    }
  }

  // trasformo il contorno comune in feature di f_dest
  if (e1 != E0_ptr)
  {
    f_dest->add_f_inside_edge(e1);
    if (e2 != E0_ptr)
      f_dest->add_f_inside_edge(e2);
  }
  else
  {
    if (e2 != E0_ptr)
      f_dest->add_f_inside_edge(e2);
  }
  e_type * ek;
  e_list_type b_el;
  get_boundary_edges(f2, f2->bounding_edge(), b_el);

  ei_type ei_end = b_el.end();
  for (ei_type ei = b_el.begin(); ei != ei_end; ei++)
  {
    ek = * ei;
    if (ek->left_face() == f2)
      ek->assign_left_face(f_dest);
     if (ek->right_face() == f2)
       ek->assign_right_face(f_dest);
  }
  
  // trasformo gli spigoli in comune alle due facce in feature di f_dest.
  // Se tra loro vi era lo spigolo rappresentante la relazione BE di f_dest
  // riassegna tale relazione
  v_type * vk;
  ei_end = el.end();
  for (ei = el.begin(); ei != ei_end; ei++)
  {
     ek = * ei;
     vk = ek->source_vertex();
     if ((!is_on_boundary(vk,f_dest)) && (!f_dest->is_f_vertex(vk)))
     {
       f_dest->add_f_vertex(vk);
       vk->assign_containing_face(f_dest);
     }
     vk = ek->target_vertex();
     if ((!is_on_boundary(vk,f_dest)) && (!f_dest->is_f_vertex(vk)))
     {
       f_dest->add_f_vertex(vk);
       vk->assign_containing_face(f_dest);
     }
     
     if (ek == f_dest->bounding_edge())
     {
       e_type * e;
       if (is_on_boundary(e1->source_vertex(), f_dest))
         get_next_boundary_edge(e1->source_vertex(), e1, e);
       else
         get_next_boundary_edge(e1->target_vertex(), e1, e);
       f_dest->assign_bounding_edge(e);
     }
  }

  // aggiorno le relazioni degli spigoli incidenti in E0_ptr
  e_type * e00, * e01, * e10, * e11;
  edge_edge(E0_ptr, e00, e01, e10, e11);
  if (e01->source_vertex() == E0_ptr->source_vertex())
    e01->assign_source_edge(e00);
  else
    e01->assign_target_edge(e00);  
  if (e10->source_vertex() == E0_ptr->target_vertex())
    e10->assign_source_edge(e11);
  else
    e10->assign_target_edge(e11);

  // controlla e corregge le relazioni associate alla faccia F_ptr eliminando
  // eventuali entita' duplicate.
  refine_face_relations(F_ptr);
  
  // elimino E0_ptr dalla suddivisione
  remove_entity(E0_ptr);    
  // elimino f2 dalla suddivisione 
  remove_entity(f2);    

  F_ptr = f_dest;
  
  return true;
}
