// ****************************************************************************
//
// Il modulo definisce le funzioni di interfaccia alle relazioni complete
// associate alle entita' definite nel tipo PEG2.
//
// ****************************************************************************

// *************** INFINITY ***************************************************
// Ritorna il puntatore alla faccia infinita.

template <class VInfo, class EInfo, class FInfo>
face_2<VInfo,EInfo,FInfo> * PEG2<VInfo,EInfo,FInfo>::infinity()
{
  return infinity_face;
}

// *************** IS INFINITY FACE *******************************************
// Ritorna se la faccia indicata da F_ptr e' la faccia infinita.

template <class VInfo, class EInfo, class FInfo>
bool PEG2<VInfo,EInfo,FInfo>::is_infinity_face(f_type * F_ptr)
{
  return (F_ptr == infinity_face);
}

// *************** VERTEX VERTEX **********************************************
// Assegna ad V_list la lista di puntatori ai vertici associati alla relazione 
// vertex-vertex totale del vertice indicato da V_ptr.
// N.B. : se V_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
vertex_vertex(v_type *V_ptr, v_list_type & V_list)
{
  e_type * e = V_ptr->incident_edge();

  if (e != NULL)
  {
    e_type * e1 = e;
    do
    { 
      V_list.push_front(e1->other_vertex(V_ptr));
      e1 = e1->edge_incident_in_vertex(V_ptr);
    }
    while (e1 != e);
  }
}

// *************** VERTEX EDGE ************************************************
// Assegna ad E_list la lista di puntatori agli archi associati alla relazione
// vertex-edge totale del vertice indicato da V_ptr.
// N.B. : se E_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
vertex_edge(v_type *V_ptr, e_list_type & E_list)
{
  e_type * e = V_ptr->incident_edge();

  if (e != NULL)
  {
    e_type * e1 = e;
    do
    { 
      E_list.push_front(e1);
      e1 = e1->edge_incident_in_vertex(V_ptr);
    }
    while (e1 != e);
  }
}


// *************** VERTEX FACE ************************************************
// Assegna ad F_list la lista di puntatori alle facce associate alla relazione
// vertex-face totale del vertice indicato da V_ptr.
// Se il vertice e' isolato o appartiene ad una feature la lista e' composta 
// solo dalla faccia che contiene il vertice.
// N.B. : se F_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
vertex_face(v_type * V_ptr, f_list_type & F_list)
{

  if (V_ptr->is_isolated())
    F_list.push_front(V_ptr->containing_face());
  else
  {    
    f_type * f;
    
    e_type * e = V_ptr->incident_edge();
    if (e != NULL)
    {
      e_type * e1 = e;
      do
      { 
        if (V_ptr == e1->source_vertex())
        {
          f = e1->right_face();
          if (!f->is_marked())
          {
             F_list.push_front(f);
             f->mark();
          }
        }
        else
        {
         if (!F_list.is_in_list(e1->left_face()))
            F_list.push_front(e1->left_face());
        }
        
        e1 = e1->edge_incident_in_vertex(V_ptr);
      }
      while (e1 != e);
    }

    // smarca le facce    
    fi_type fi_end = F_list.end();
    for (fi_type fi = F_list.begin(); fi != fi_end; fi++)
      (* fi)->unmark();
  }
}

// *************** EDGE VERTEX ************************************************
// Assegna a V0_ptr e V1_ptr i puntatori ai vertici associati alla relazione
// edge-vertex totale dell'arco indicato da E_ptr (rispettivanmente il vertice
// di partenza e il vertice di destinazione dell'arco orientato).

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
edge_vertex(e_type * E_ptr, v_type *& V0_ptr, v_type *& V1_ptr)
{
  V0_ptr = E_ptr->source_vertex();
  V1_ptr = E_ptr->target_vertex();
}

// *************** EDGE EDGE **************************************************
// Assegna a E00_ptr, E01_ptr, E10_ptr e E11_ptr i puntatori agli archi 
// associati alla relazione edge-edge totale dell'arco indicato da E_ptr.
// Piu' in dettaglio si ha che :
// E00_ptr e E01_ptr indicano gli archi incidenti il vertice di partenza 
// che si incontrano ruotando rispettivamente in senso antiorario e orario
// sull'arco orientato.
// E10_ptr e E11_ptr indicano gli archi incidenti il vertice di destinazione 
// che si incontrano ruotando rispettivamente in senso orario ed antiorario
// sull'arco orientato.
 
template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
edge_edge(e_type * E_ptr, e_type *& E00_ptr, e_type *& E01_ptr,
	  e_type *& E10_ptr, e_type *& E11_ptr)
{
  E00_ptr = E01_ptr = E10_ptr = E11_ptr = NULL;
  
  if (E_ptr != NULL)
  {
    v_type * vk = E_ptr->source_vertex();
    E00_ptr = E01_ptr = E_ptr->edge_incident_in_vertex(vk);
    while (E01_ptr->edge_incident_in_vertex(vk) != E_ptr)
      E01_ptr = E01_ptr->edge_incident_in_vertex(vk);
      
    vk = E_ptr->target_vertex();
    E10_ptr = E11_ptr = E_ptr->edge_incident_in_vertex(vk);
    while (E10_ptr->edge_incident_in_vertex(vk) != E_ptr)
      E10_ptr = E10_ptr->edge_incident_in_vertex(vk);
  }
}

// *************** EDGE FACE **************************************************
// Assegna a F0_ptr e F1_ptr i puntatori alle facce associate alla relazione
// edge-face totale dell'arco indicato da E_ptr (rispettivamente la faccia
// che giace alla sinistra e la faccia che giace alla destra dell'arco
// orientato.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
edge_face(e_type * E_ptr, f_type *& F0_ptr, f_type *& F1_ptr)
{
  F0_ptr = E_ptr->left_face();
  F1_ptr = E_ptr->right_face();
}

// *************** FACE VERTEX ************************************************
// Assegna a V_list_list la lista di liste di puntatori ai vertici associati
// alla relazione face-vertex totale della faccia indicata da F_ptr. (una lista
// per il contorno esterno di F_ptr piu' una lista per ogni suo ciclo interno).
// N.B. : se V_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
face_vertex(f_type * F_ptr, list_2<Vertex_List> & V_list_list)
{
  e_type * e;
  v_list_type V_list;
  
  // vertici del boundary
  if (!is_infinity_face(F_ptr))
  {
    e = F_ptr->bounding_edge();
    get_boundary_vertices(F_ptr, e, V_list);
    V_list_list.push_back(V_list);
  }
  
  // aggiungo i vertici dei cicli interni
  ei_type ei_end = F_ptr->inside_edges_end();  
  for (ei_type ei = F_ptr->inside_edges_begin(); ei != ei_end; ei++)
  {
    e = * ei;
    V_list.free();
    get_boundary_vertices(F_ptr, e, V_list);
    V_list_list.push_back(V_list);
  }
}

// *************** FACE EDGE **************************************************
// Assegna a E_list_list la lista di liste di puntatori agli archi associati
// alla relazione face-face totale della faccia indicata da F_ptr. (una lista
// per il contorno esterno di F_ptr piu' una lista per ogni suo ciclo interno).
// N.B. : se E_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
face_edge(f_type * F_ptr, list_2<Edge_List> & E_list_list)
{
  e_type * e;
  e_list_type E_list;

  // facce associate al boundary
  if (!is_infinity_face(F_ptr))
  {
    e = F_ptr->bounding_edge();
    get_boundary_edges(F_ptr, e, E_list);
    E_list_list.push_back(E_list);
  }
  
  // aggiungo gli edge dei cicli interni (buchi)
  ei_type ei_end = F_ptr->inside_edges_end();  
  for (ei_type ei = F_ptr->inside_edges_begin(); ei != ei_end; ei++)
  {
    e = * ei;
    E_list.free();
    get_boundary_edges(F_ptr, e, E_list);
    E_list_list.push_back(E_list);
  }
}

// *************** FACE FACE **************************************************
// Assegna a F_list_list la lista di liste di puntatori alle facce associate
// alla relazione face-face totale della faccia indicata da F_ptr (una lista
// per il contorno esterno di F_ptr piu' una lista per ogni suo ciclo interno).
// N.B. : se F_list_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::
face_face(f_type * F_ptr, list_2<Face_List> & F_list_list)
{
  e_type * e;
  f_list_type F_list;

  // facce adiacenti al boundary
  if (!is_infinity_face(F_ptr))
  {
    e = F_ptr->bounding_edge();
    get_boundary_faces(F_ptr, e, F_list);
    F_list_list.push_back(F_list);
  }
  
  // aggiungo le facce adiacenti ai cicli interni
  ei_type ei_end = F_ptr->inside_edges_end();  
  for (ei_type ei = F_ptr->inside_edges_begin(); ei != ei_end; ei++)
  {
    e = * ei;
    F_list.free();
    get_boundary_faces(F_ptr, e, F_list);
    F_list_list.push_back(F_list);
  }
}

// ******************** F FACE VERTEX *****************************************
// Assegna a V_list la lista dei vertici isolati e dei vertici appartenenti a
// feature, contenuti nella faccia indicata da F_ptr.
// N.B. : se V_list non e' vuota inserisce in testa alla lista.

template <class VInfo, class EInfo, class FInfo>
void
PEG2<VInfo,EInfo,FInfo>::f_face_vertex(Face_2 * F_ptr, Vertex_List & V_list)
{
  Vertex_Iterator end_it = F_ptr->f_vertices_end();
  for (Vertex_Iterator vi = F_ptr->f_vertices_begin(); vi != end_it; vi++)
    V_list.push_front(* vi);
}

  
// ******************** F FACE EDGE *******************************************
// E_list_list rappresenta una lista di liste di puntatori ad archi e viene
// assegnata con liste di puntatori agli archi contenuti nella faccia F_ptr 
// (una lista per ogni componente connessa (feature) memorizzata nella relazione
// parziale f-inside-edges).
// Per non marcare i vertici, utilizza un metodo di liste affiancate (lista vl
// e lista twin).
// In questo algoritmo due spigoli separati da una faccia chiusa non vengono
// considerati connessi. Non e' comunque un problema estenderlo nel caso si
// desiderasse il contrario.

template <class VInfo, class EInfo, class FInfo>
void PEG2<VInfo,EInfo,FInfo>::f_face_edge(Face_2 * F_ptr,
				          list_2<Edge_List> & E_list_list)
{
  Edge_Iterator ei_end = F_ptr->f_inside_edges_end();  
  for (Edge_Iterator ei = F_ptr->f_inside_edges_begin(); ei != ei_end; ei++)
  {
    v_list_type v_stack;
    e_list_type e_stack;
    v_type * v;
    e_type * e = * ei;

    if (e != NULL) 
    {
      e_list_type el;
      
      v = e->source_vertex();
      e_type * e1 = e;
      do
      {
        if (e1->is_in_feature(F_ptr)) 
        {
          el.push_front(e1);
          e_stack.push_back(e1);
          v_stack.push_back(e1->other_vertex(v));
        }
        e1 = e1->edge_incident_in_vertex(v);
      }
      while (e1 != e);
      
      while(!e_stack.empty())
      {
        e = e_stack.front();
        v = v_stack.front();
        e_stack.pop_front();
        v_stack.pop_front();
        e1 = e->edge_incident_in_vertex(v);
        while (e1 != e)
        {
          if (e1->is_in_feature(F_ptr)) 
          {
            el.push_front(e1);
            e_stack.push_back(e1);
            v_stack.push_back(e1->other_vertex(v));
          }
          e1 = e1->edge_incident_in_vertex(v);
        }
      }
      E_list_list.push_back(el);
    }
  }
}
